package edu.uoc.bsolee.sarau.adapters

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import edu.uoc.bsolee.sarau.databinding.ItemAddTicketBinding
import edu.uoc.bsolee.sarau.model.Ticket
import java.text.DecimalFormat
import java.text.DecimalFormatSymbols
import java.text.NumberFormat


class TicketAdapter(
    private val saveClickListener: TicketClickListener,
    private val deleteClickListener: TicketClickListener
) : ListAdapter<Ticket, TicketAdapter.TicketViewHolder>(TicketsDiffCallBack) {

    companion object TicketsDiffCallBack : DiffUtil.ItemCallback<Ticket>() {
        override fun areItemsTheSame(oldItem: Ticket, newItem: Ticket): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: Ticket, newItem: Ticket): Boolean {
            return oldItem == newItem
        }

    }

    class TicketViewHolder(
        private val binding: ItemAddTicketBinding,
        private val saveClickListener: TicketClickListener,
        private val deleteClickListener: TicketClickListener
    ) : RecyclerView.ViewHolder(binding.root) {
        private lateinit var item: Ticket

        init {
            binding.btnSave.setOnClickListener {
                setNotEditableUi()
                saveClickListener.onClick(getTicket())
            }

            binding.btnDelete.setOnClickListener {
                deleteClickListener.onClick(getTicket())
            }
        }

        private fun getTicket(): Ticket {
            val formatter: NumberFormat = DecimalFormat.getInstance()
            val separator = DecimalFormatSymbols.getInstance().decimalSeparator
            (formatter as DecimalFormat).isParseBigDecimal = true
            val price = binding.etTicketPrice.text.toString().let {
                if (it.isNotEmpty()) {
                    val newString = it.replace(".", separator.toString())
                    return@let formatter.parse(newString).toDouble()
                } else {
                    return@let null
                }
            }
            val numTickets = binding.etTicketNum.text.toString().let {
                if (it.isNotEmpty()) {
                    return@let it.toInt()
                } else {
                    return@let null
                }
            }
            return Ticket(
                item.id, binding.etTicketName.text.toString(),
                binding.etTicketDescription.text.toString(),
                price,
                numTickets, true
            )
        }


        fun bindTo(ticket: Ticket) {
            item = ticket
            binding.apply {
                item.name?.let { etTicketName.setText(it) }
                item.description?.let { etTicketDescription.setText(it) }
                item.price?.let { etTicketPrice.setText(NumberFormat.getInstance().format(it)) }
                item.capacity?.let { etTicketNum.setText("$it") }
                btnEdit.setOnClickListener {
                    setEditableUi()
                }
                if (etTicketName.text.isNullOrEmpty() ||
                    etTicketDescription.text.isNullOrEmpty() ||
                    etTicketPrice.text.isNullOrEmpty() ||
                    etTicketNum.text.isNullOrEmpty()
                ) {
                    setEditableUi()
                } else {
                    setNotEditableUi()
                }
            }
        }

        private fun setEditableUi() {
            binding.apply {
                etTicketName.isEnabled = true
                etTicketDescription.isEnabled = true
                etTicketPrice.isEnabled = true
                etTicketNum.isEnabled = true
                btnSave.visibility = View.VISIBLE
                btnDelete.visibility = View.VISIBLE
                btnEdit.visibility = View.GONE
            }
        }

        private fun setNotEditableUi() {
            binding.apply {
                etTicketName.isEnabled = false
                etTicketDescription.isEnabled = false
                etTicketPrice.isEnabled = false
                etTicketNum.isEnabled = false
                btnSave.visibility = View.GONE
                btnDelete.visibility = View.GONE
                btnEdit.visibility = View.VISIBLE
            }
        }
    }


    class TicketClickListener(val clickListener: (ticket: Ticket) -> Unit) {
        fun onClick(ticket: Ticket) = clickListener(ticket)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): TicketViewHolder {
        return TicketViewHolder(
            ItemAddTicketBinding.inflate(
                LayoutInflater.from(parent.context),
                parent,
                false
            ), saveClickListener, deleteClickListener
        )
    }

    override fun onBindViewHolder(holder: TicketViewHolder, position: Int) {
        holder.bindTo(getItem(position))
    }
}
