package edu.uoc.bsolee.sarau.repository

import android.graphics.Bitmap
import android.net.Uri
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.google.firebase.firestore.FirebaseFirestore
import com.google.firebase.ktx.Firebase
import com.google.firebase.storage.ktx.storage
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.model.*
import java.io.ByteArrayOutputStream
import java.util.*

class EventRepository {

    companion object {
        const val COLLECTION_EVENTS = "Events"
        const val SUBCOLLECTION_TICKETS = "Tickets"
        const val SUBCOLLECTION_PROMOTERS = "Promoters"
        const val FIELD_START_DATE = "startDate"
        const val FIELD_HOST = "host"
    }

    private val db = FirebaseFirestore.getInstance()
    private val storage = Firebase.storage
    private val imageUriLiveData = MutableLiveData<Uri>()
    private val isSuccessfullySaved = MutableLiveData<Boolean>()
    private val isSuccessfullyUpdated = MutableLiveData<Boolean>()
    private val eventLiveData = MutableLiveData<Event>()
    private val ticketsLiveData = MutableLiveData<List<Ticket>>()
    private val promotersLiveData = MutableLiveData<List<Promoter>>()
    private val nextEventsLiveData = MutableLiveData<List<Event>>()
    private val lastEventsLiveData = MutableLiveData<List<Event>>()
    private val nextUserEventsLiveData = MutableLiveData<List<Event>>()


    fun saveImageToStorage(name: String, image: Bitmap) {
        val imageRef = storage.reference.child("$COLLECTION_EVENTS/$name/poster.jpg")
        val baos = ByteArrayOutputStream()
        image.compress(Bitmap.CompressFormat.JPEG, 100, baos)
        val data = baos.toByteArray()

        val uploadTask = imageRef.putBytes(data)

        uploadTask.continueWithTask { task ->
            if (!task.isSuccessful) {
                task.exception?.let {
                    throw it
                }
            }
            imageRef.downloadUrl
        }.addOnCompleteListener { task ->
            if (task.isSuccessful) {
                val downloadUri = task.result
                imageUriLiveData.postValue(downloadUri)
            }
        }
    }

    fun getImageUriLiveData(): LiveData<Uri> = imageUriLiveData
    fun getIsEventSaved(): LiveData<Boolean> = isSuccessfullySaved

    fun saveEvent(event: Event, image: Bitmap?, tickets: List<Ticket>, promoters: List<Promoter>) {
        if (image != null) {
            // Save image
            val imageRef = storage.reference.child("$COLLECTION_EVENTS/${event.name}/poster.jpg")
            val baos = ByteArrayOutputStream()
            image.compress(Bitmap.CompressFormat.JPEG, 100, baos)
            val data = baos.toByteArray()

            val uploadTask = imageRef.putBytes(data)

            uploadTask.continueWithTask { task ->
                if (!task.isSuccessful) {
                    task.exception?.let {
                        throw it
                    }
                }
                imageRef.downloadUrl
            }.addOnCompleteListener { task ->
                if (task.isSuccessful) {
                    val downloadUri = task.result
                    saveEventToFirestore(downloadUri, event, tickets, promoters)
                }
            }
        } else {
            saveEventToFirestore(null, event, tickets, promoters)
        }
    }

    private fun saveEventToFirestore(
        uri: Uri?,
        event: Event,
        tickets: List<Ticket>,
        promoters: List<Promoter>
    ) {
        val docRef = db.collection(COLLECTION_EVENTS).document(event.id)
        docRef.set(event.changeImageUri(uri?.toString())).addOnSuccessListener {
            var areTicketsSaved = true
            var arePromotersSaved = true
            tickets.forEach {
                docRef.collection(SUBCOLLECTION_TICKETS).document(it.id).set(it)
                    .addOnFailureListener {
                        areTicketsSaved = false
                    }
            }
            promoters.forEach {
                docRef.collection(SUBCOLLECTION_PROMOTERS).document(it.id).set(it)
                    .addOnFailureListener {
                        arePromotersSaved = false
                    }
            }
            isSuccessfullySaved.value = areTicketsSaved && arePromotersSaved
        }
    }

    fun getEventFromId(eventId: String) {
        db.collection(COLLECTION_EVENTS).document(eventId).get().addOnSuccessListener {
            val event = it.toObject(Event::class.java)
            event?.let { itEvent -> eventLiveData.value = itEvent }
            getEventTicketsFromId(eventId)
            getEventPromotersFromId(eventId)
        }
    }

    fun getEventLiveData(): LiveData<Event> = eventLiveData

    private fun getEventTicketsFromId(evenId: String) {
        db.collection(COLLECTION_EVENTS).document(evenId).collection(SUBCOLLECTION_TICKETS).get()
            .addOnSuccessListener {
                val documents = it.documents
                val tickets = mutableListOf<Ticket>()
                for (doc in documents) {
                    val ticket = doc.toObject(Ticket::class.java)
                    ticket?.let { itTicket -> tickets.add(itTicket) }
                }
                ticketsLiveData.value = tickets
            }
    }

    private fun getEventPromotersFromId(evenId: String) {
        db.collection(COLLECTION_EVENTS).document(evenId).collection(SUBCOLLECTION_PROMOTERS).get()
            .addOnSuccessListener {
                val documents = it.documents
                val promoters = mutableListOf<Promoter>()
                for (doc in documents) {
                    val promoter = doc.toObject(Promoter::class.java)
                    promoter?.let { itPromoter -> promoters.add(itPromoter) }
                }
                promotersLiveData.value = promoters
            }
    }

    fun getTicketsLiveData() = ticketsLiveData

    fun getPromotersLiveData() = promotersLiveData


    fun getHostNextEvents() {
        db.collection(COLLECTION_EVENTS).whereEqualTo(FIELD_HOST, AppSarau.prefs.getUser()).get()
            .addOnSuccessListener {
                val documents = it.documents
                val events = mutableListOf<Event>()
                for (doc in documents) {
                    val event = doc.toObject(Event::class.java)
                    event?.let { itEvent -> if (itEvent.startDate >= Date()) events.add(itEvent) }
                }
                nextEventsLiveData.value = events.sortedBy { item -> item.startDate }
            }
    }

    fun getHostLastEvents() {
        db.collection(COLLECTION_EVENTS).whereEqualTo(FIELD_HOST, AppSarau.prefs.getUser()).get()
            .addOnSuccessListener {
                val documents = it.documents
                val events = mutableListOf<Event>()
                for (doc in documents) {
                    val event = doc.toObject(Event::class.java)
                    event?.let { itEvent -> if (itEvent.startDate < Date()) events.add(itEvent) }
                }
                lastEventsLiveData.value = events.sortedBy { item -> item.startDate }
            }
    }

    fun getNextEventsLiveData() = nextEventsLiveData

    fun getLastEventsLiveData() = lastEventsLiveData

    fun updateEvent(event: Event, image: Bitmap?, tickets: List<Ticket>, promoters: List<Promoter>) {
        if (image != null) {
            // Save image
            val imageRef = storage.reference.child("$COLLECTION_EVENTS/${event.name}/poster.jpg")
            val baos = ByteArrayOutputStream()
            image.compress(Bitmap.CompressFormat.JPEG, 100, baos)
            val data = baos.toByteArray()

            val uploadTask = imageRef.putBytes(data)

            uploadTask.continueWithTask { task ->
                if (!task.isSuccessful) {
                    task.exception?.let {
                        throw it
                    }
                }
                imageRef.downloadUrl
            }.addOnCompleteListener { task ->
                if (task.isSuccessful) {
                    val downloadUri = task.result
                    updateEventToFirestore(downloadUri, event, tickets, promoters)
                }
            }
        } else {
            updateEventToFirestore(Uri.parse(event.image), event, tickets, promoters)
        }
    }

    private fun updateEventToFirestore(
        uri: Uri?,
        event: Event,
        tickets: List<Ticket>,
        promoters: List<Promoter>
    ) {
        val docRef = db.collection(COLLECTION_EVENTS).document(event.id)
        docRef.set(event.changeImageUri(uri?.toString())).addOnSuccessListener {
            var areTicketsSaved = true
            var arePromotersSaved = true
            tickets.forEach {
                docRef.collection(SUBCOLLECTION_TICKETS).document(it.id).set(it)
                    .addOnFailureListener {
                        areTicketsSaved = false
                    }
            }
            promoters.forEach {
                docRef.collection(SUBCOLLECTION_PROMOTERS).document(it.id).set(it)
                    .addOnFailureListener {
                        arePromotersSaved = false
                    }
            }
            isSuccessfullyUpdated.value = areTicketsSaved && arePromotersSaved
        }
    }

    fun getIsEventUpdated(): LiveData<Boolean> = isSuccessfullyUpdated

    fun getUsersNextEvents() {
        db.collection(COLLECTION_EVENTS).whereGreaterThanOrEqualTo(FIELD_START_DATE, Date()).limit(25L).get()
            .addOnSuccessListener {
                val documents = it.documents
                val events = mutableListOf<Event>()
                for (doc in documents) {
                    val event = doc.toObject(Event::class.java)
                    event?.let { itEvent -> if (itEvent.startDate >= Date()) events.add(itEvent) }
                }
                nextUserEventsLiveData.value = events.sortedBy { item -> item.startDate }
            }
    }
    fun getNextUserEventsLiveData() : LiveData<List<Event>> = nextUserEventsLiveData


}