package edu.uoc.bsolee.sarau.ui.host.check_in

import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanIntentResult
import com.journeyapps.barcodescanner.ScanOptions
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.databinding.ActivityEventCheckInBinding
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.checkIn
import java.text.NumberFormat
import java.text.SimpleDateFormat
import java.util.*


class EventCheckInActivity : AppCompatActivity() {

    companion object {
        const val EVENT_ID = "eventId"
    }


    // Argument
    private lateinit var eventId: String

    // View Binding
    private var _binding: ActivityEventCheckInBinding? = null
    val binding: ActivityEventCheckInBinding get() = _binding!!

    // ViewModel
    private val viewModel: EventCheckInViewModel by viewModels { EventCheckInViewModelFactory() }


    // Launcher
    private val barcodeLauncher: ActivityResultLauncher<ScanOptions?> = registerForActivityResult(
        ScanContract()
    ) { result: ScanIntentResult ->
        if (result.contents == null) {
            Toast.makeText(this, "Cancelled", Toast.LENGTH_SHORT).show()
        } else {
            viewModel.setQR(result.contents)
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        eventId = intent.getStringExtra(EVENT_ID).toString()
        _binding = ActivityEventCheckInBinding.inflate(layoutInflater)
        initUi()
        initObservers()
        setContentView(binding.root)
    }

    override fun onResume() {
        super.onResume()
        viewModel.loadEvent(eventId)
    }

    private fun initUi() {
        // QR Button
        binding.btnCheckIn.setOnClickListener {
            hideResult()
            val options = ScanOptions()
            options.apply {
                setDesiredBarcodeFormats(ScanOptions.QR_CODE)
                setPrompt(getString(R.string.QR_prompt))
                setCameraId(0) // Use back camera of the device
                setBeepEnabled(true)
                setOrientationLocked(false)
                setBarcodeImageEnabled(false)
            }
            barcodeLauncher.launch(options)
        }

    }

    private fun initObservers() {
        viewModel.promoters.observe(this) {
            viewModel.loadBookings(it)
        }

        viewModel.bookings.observe(this) {
            populateUI(it)
        }

        viewModel.qrCode.observe(this) {
            it?.let { qr ->
                viewModel.resetQr()
                viewModel.checkScanResult(qr)
            }
        }

        viewModel.bookingNotExist.observe(this) {
            if (it) {
                viewModel.resetCheckIn()
                showNotExistResult()
            }
        }

        viewModel.correctCheckIn.observe(this) {
            it?.let {
                viewModel.resetCheckIn()
                viewModel.updateBookingToDB(it.checkIn())
                showCorrectCheckIn(it)
            }
        }

        viewModel.checkedIn.observe(this) {
            it?.let {
                viewModel.resetCheckIn()
                showCheckedInResult(it)
            }
        }
    }


    private fun populateUI(list: List<Booking>) {
        val totalBookings = list.filter { it.paid }.size
        var checkedIn = 0
        var pending = 0
        for (booking in list.filter { it.paid }) {
            if (booking.checkedIn) {
                checkedIn += 1
            } else {
                pending += 1
            }
        }
        title = viewModel.event.value?.name ?: getString(R.string.check_in)
        binding.apply {
            tvTotalBookingsValue.text = totalBookings.toString()
            tvCheckedInValue.text = checkedIn.toString()
            tvPendingValue.text = pending.toString()
        }
    }

    private fun showNotExistResult() {
        binding.apply {
            clResult.visibility = View.VISIBLE
            clResult.background = ContextCompat.getDrawable(baseContext, R.color.red)
            tvName.text = ""
            tvDescription.text = ""
            tvPrice.text = ""
            tvResult.text = getString(R.string.booking_not_exist_message)
        }
    }

    private fun showCheckedInResult(booking: Booking) {
        val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
        val dateFormat = SimpleDateFormat("dd/MM/yy - HH:mm", Locale.getDefault())
        val message = StringBuilder()
        message.append(getString(R.string.booking_checked_in_message))
        message.append(" ")
        booking.dateCheckIn?.let { message.append(dateFormat.format(it)) }
        binding.apply {
            clResult.visibility = View.VISIBLE
            clResult.background = ContextCompat.getDrawable(baseContext, R.color.red)
            tvName.text = booking.name
            tvDescription.text = booking.description
            tvPrice.text = numFormat.format(booking.amount)
            tvResult.text = message.toString()
        }
    }

    private fun showCorrectCheckIn(booking: Booking) {
        val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
        binding.apply {
            clResult.visibility = View.VISIBLE
            clResult.background = ContextCompat.getDrawable(baseContext, R.color.green)
            tvName.text = booking.name
            tvDescription.text = booking.description
            tvPrice.text = numFormat.format(booking.amount)
            tvResult.text = getString(R.string.booking_correct_message)
        }
    }

    private fun hideResult() {
        binding.clResult.visibility = View.GONE
    }
}