package edu.uoc.bsolee.sarau.ui.host.create_event

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.android.material.datepicker.MaterialDatePicker
import com.google.android.material.timepicker.MaterialTimePicker
import com.google.android.material.timepicker.TimeFormat
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.BuildConfig
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.PromoterAdapter
import edu.uoc.bsolee.sarau.adapters.TicketAdapter
import edu.uoc.bsolee.sarau.databinding.ActivityCreateEventBinding
import edu.uoc.bsolee.sarau.model.*
import edu.uoc.bsolee.sarau.ui.host.detail_event.HostDetailEventActivity
import edu.uoc.bsolee.sarau.utils.Utils
import java.text.SimpleDateFormat
import java.util.*

class CreateEventActivity : AppCompatActivity() {


    // ViewModel
    private val viewModel: CreateEventViewModel by viewModels { CreateEventViewModelFactory() }

    // ViewBinding
    private var _binding: ActivityCreateEventBinding? = null
    private val binding: ActivityCreateEventBinding
        get() = _binding!!

    private val launcher =
        registerForActivityResult(ActivityResultContracts.GetContent()) { result ->
            viewModel.selectImage(result)
        }

    // adapters
    private val ticketAdapter: TicketAdapter = TicketAdapter(
        saveClickListener = TicketAdapter.TicketClickListener {
            viewModel.saveTicket(it)
        }, deleteClickListener = TicketAdapter.TicketClickListener {
            viewModel.deleteTicket(it.id)
        })

    private val promoterAdapter: PromoterAdapter =
        PromoterAdapter(saveClickListener = PromoterAdapter.PromoterClickListener {
            viewModel.savePromoter(it)
        }, deleteClickListener = PromoterAdapter.PromoterClickListener {
            viewModel.deletePromoter(it.id)
        })

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        _binding = ActivityCreateEventBinding.inflate(layoutInflater)
        setContentView(binding.root)
        title = getString(R.string.create_event)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        if (BuildConfig.DEBUG) {
            populateUi(
                Event(
                    "Evento de Prueba",
                    "Descripción del evento de prueba",
                    AppSarau.prefs.getUser(),
                    Address(
                        "Santa Marta 13",
                        "08030", "Barcelona", "Spain"
                    ),
                    Calendar.getInstance().time,
                    Calendar.getInstance().time,
                    mutableListOf(),
                    false,
                    null,
                    PaymentMethod(
                        "Transferencia bancaria",
                        "Transferencia bancaria indicando el nombre y la referencia de la reseva",
                        "ES152345234523452345"
                    )
                )
            )
        }
        initUi()
        initObservers()
    }

    private fun initUi() {
        // init RecyclerViews
        binding.rvTickets.layoutManager = LinearLayoutManager(this)
        binding.rvTickets.apply {
            adapter = ticketAdapter
            setHasFixedSize(false)
            isNestedScrollingEnabled = false
        }

        binding.rvPromoters.layoutManager = LinearLayoutManager(this)
        binding.rvPromoters.apply {
            adapter = promoterAdapter
            setHasFixedSize(false)
            isNestedScrollingEnabled = false
        }

        // Listeners
        binding.btnAddTicket.setOnClickListener {
            viewModel.addTicket(
                Ticket(
                    "tk-${viewModel.ticketsListSize() + 1}", null, null, null, null, true
                )
            )
        }
        binding.btnAddPromoter.setOnClickListener {
            viewModel.addPromoter(
                Promoter("prm-${viewModel.promotersListSize() + 1}", null, null, null, null, true)
            )
        }

        binding.tvStartDate.setOnClickListener {
            showStartDatePicker()
        }

        binding.tvStartTime.setOnClickListener {
            showStartDatePicker()
        }

        binding.tvEndDate.setOnClickListener {
            showEndDatePicker()
        }

        binding.tvEndTime.setOnClickListener {
            showEndDatePicker()
        }

        binding.btnAddPoster.setOnClickListener {
            launcher.launch("image/*")
        }

        binding.btnSaveDraft.setOnClickListener {
            val event = Event(
                name = binding.etEventName.text.toString(),
                description = binding.etDescription.text.toString(),
                host = AppSarau.prefs.getUser(),
                address = Address(
                    street = binding.etAddress.text.toString(),
                    postalCode = binding.etPostalCode.text.toString(),
                    city = binding.etCity.text.toString(),
                    country = binding.etCountry.text.toString()
                ),
                viewModel.startDate.value ?: Date(0),
                viewModel.endDate.value ?: Date(0),
                listOf(),
                false,
                viewModel.imageTempUri.value.toString(),
                PaymentMethod(
                    name = binding.etPaymentMethodName.text.toString(),
                    description = binding.etPaymentMethodDescription.text.toString(),
                    account = binding.etPaymentMethodNum.text.toString()
                )
            )
            viewModel.validateDraft(event)
        }

        binding.btnPublish.setOnClickListener {
            val event = Event(
                name = binding.etEventName.text.toString(),
                description = binding.etDescription.text.toString(),
                host = AppSarau.prefs.getUser(),
                address = Address(
                    street = binding.etAddress.text.toString(),
                    postalCode = binding.etPostalCode.text.toString(),
                    city = binding.etCity.text.toString(),
                    country = binding.etCountry.text.toString()
                ),
                viewModel.startDate.value ?: Date(0),
                viewModel.endDate.value ?: Date(0),
                listOf(),
                true,
                viewModel.imageTempUri.value.toString(),
                PaymentMethod(
                    name = binding.etPaymentMethodName.text.toString(),
                    description = binding.etPaymentMethodDescription.text.toString(),
                    account = binding.etPaymentMethodNum.text.toString()
                )
            )
            viewModel.validatePublish(event)
        }
    }

    private fun initObservers() {
        viewModel.tickets.observe(this) {

            ticketAdapter.submitList(it.filter { ticket -> ticket.active })
        }

        viewModel.promoters.observe(this) {
            promoterAdapter.submitList(it.filter { promoter -> promoter.active })
        }

        viewModel.imageTempUri.observe(this) {
            binding.tvMessageImage.visibility = View.VISIBLE
            convertToBitmap(it)
        }

        viewModel.startDate.observe(this) { date ->
            date?.let {
                val timeZone = TimeZone.getTimeZone("GMT")
                val dateFormat = SimpleDateFormat("dd/MM/yyy", Locale.getDefault())
                val timeFormat = SimpleDateFormat("HH:mm", Locale.getDefault())
                dateFormat.timeZone = timeZone
                timeFormat.timeZone = timeZone

                binding.tvStartDate.text = dateFormat.format(it)
                binding.tvStartTime.text = timeFormat.format(it)
            }
        }

        viewModel.endDate.observe(this) { date ->
            date?.let {
                val timeZone = TimeZone.getTimeZone("GMT")
                val dateFormat = SimpleDateFormat("dd/MM/yyy", Locale.getDefault())
                val timeFormat = SimpleDateFormat("HH:mm", Locale.getDefault())
                dateFormat.timeZone = timeZone
                timeFormat.timeZone = timeZone

                binding.tvEndDate.text = dateFormat.format(it)
                binding.tvEndTime.text = timeFormat.format(it)
            }
        }

        viewModel.isEventSaved.observe(this) {
            if (it) {
                Toast.makeText(this, getString(R.string.event_saved), Toast.LENGTH_LONG).show()
                navigateToHostDetailActivity()
            }
        }

        viewModel.errorMessage.observe(this) {
            if (it != null) {
                viewModel.resetErrorMessage()
                AlertDialog.Builder(this)
                    .setTitle(R.string.error)
                    .setMessage(it)
                    .setPositiveButton("Ok") { dialog, _ -> dialog.dismiss() }
                    .create()
                    .show()
            }
        }

        viewModel.validatedDraft.observe(this) {
            if (it != null) {
                viewModel.resetValidatedDraft()
                viewModel.saveEvent(it)
            }
        }

        viewModel.validatedPublish.observe(this) {
            if (it != null) {
                viewModel.resetValidatePublish()
                viewModel.saveEvent(it)
            }
        }
    }

    private fun showStartDatePicker() {
        val datePicker = MaterialDatePicker.Builder.datePicker()
            .setSelection(MaterialDatePicker.todayInUtcMilliseconds())
            .setTitleText(getString(R.string.select_start_date)).build()

        datePicker.show(supportFragmentManager, "MATERIAL_DATE_PICKER")

        datePicker.addOnPositiveButtonClickListener {
            viewModel.setStartDate(it)
            showStartTimePicker()
        }

    }

    private fun showStartTimePicker() {
        val timePicker =
            MaterialTimePicker.Builder().setTitleText(getString(R.string.select_start_time))
                .setTimeFormat(TimeFormat.CLOCK_24H).build()
        timePicker.show(supportFragmentManager, "CreateEventActivity")

        timePicker.addOnPositiveButtonClickListener {
            viewModel.setStartTime(timePicker.hour, timePicker.minute)
        }
    }

    private fun showEndDatePicker() {
        val datePicker = MaterialDatePicker.Builder.datePicker()
            .setSelection(MaterialDatePicker.todayInUtcMilliseconds())
            .setTitleText(getString(R.string.select_start_date)).build()

        datePicker.show(supportFragmentManager, "MATERIAL_DATE_PICKER")

        datePicker.addOnPositiveButtonClickListener {
            viewModel.setEndDate(it)
            showEndTimePicker()
        }

    }

    private fun showEndTimePicker() {
        val timePicker =
            MaterialTimePicker.Builder().setTitleText(getString(R.string.select_start_time))
                .setTimeFormat(TimeFormat.CLOCK_24H).build()
        timePicker.show(supportFragmentManager, "CreateEventActivity")

        timePicker.addOnPositiveButtonClickListener {
            viewModel.setEndTime(timePicker.hour, timePicker.minute)
        }
    }

    private fun convertToBitmap(imageUri: Uri) {
        val bitmap = Utils.convertToBitmap(applicationContext, imageUri)
        viewModel.setPosterBitmap(bitmap)
    }

    private fun navigateToHostDetailActivity() {
        val intent = Intent(baseContext, HostDetailEventActivity::class.java)
        intent.putExtra(HostDetailEventActivity.EVENT_ID, viewModel.eventId)
        startActivity(intent)
        finish()
    }

    private fun populateUi(event: Event) {
        binding.apply {
            etEventName.setText(event.name)
            etAddress.setText(event.address.street)
            etPostalCode.setText(event.address.postalCode)
            etCity.setText(event.address.city)
            etCountry.setText(event.address.country)
            etDescription.setText(event.description)
            etPaymentMethodName.setText(event.paymentMethod.name)
            etPaymentMethodDescription.setText(event.paymentMethod.description)
            etPaymentMethodNum.setText(event.paymentMethod.account)

            val startDate = Utils.setDateTime(event.startDate)
            tvStartDate.text = startDate.first
            tvStartTime.text = startDate.second

            val endDate = Utils.setDateTime(event.endDate)
            tvEndDate.text = endDate.first
            tvEndTime.text = endDate.second
        }
    }
}