package edu.uoc.bsolee.sarau.ui.host.detail_event

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.ScrollView
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import com.bumptech.glide.Glide
import com.google.android.material.datepicker.MaterialDatePicker
import com.google.android.material.timepicker.MaterialTimePicker
import com.google.android.material.timepicker.TimeFormat
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.PromoterAdapter
import edu.uoc.bsolee.sarau.adapters.TicketAdapter
import edu.uoc.bsolee.sarau.databinding.ActivityHostDetailEventBinding
import edu.uoc.bsolee.sarau.model.*
import edu.uoc.bsolee.sarau.ui.host.booking_list.BookingListActivity
import edu.uoc.bsolee.sarau.ui.host.check_in.EventCheckInActivity
import edu.uoc.bsolee.sarau.utils.Utils
import java.text.SimpleDateFormat
import java.util.*

class HostDetailEventActivity : AppCompatActivity() {

    companion object {
        const val EVENT_ID = "eventId"
    }

    // Argument
    private lateinit var eventId: String

    // ViewModel
    private val viewModel: HostDetailEventViewModel by viewModels { HostDetailEventViewModelFactory() }

    // ViewBinding
    private var _binding: ActivityHostDetailEventBinding? = null
    private val binding: ActivityHostDetailEventBinding
        get() = _binding!!

    private val launcher =
        registerForActivityResult(ActivityResultContracts.GetContent()) { result ->
            viewModel.selectImage(result)
        }

    // adapters
    private val ticketAdapter: TicketAdapter = TicketAdapter(
        saveClickListener = TicketAdapter.TicketClickListener {
            viewModel.saveTicket(it)
        }, deleteClickListener = TicketAdapter.TicketClickListener {
            viewModel.deleteTicket(it.id)
        })

    private val promoterAdapter: PromoterAdapter =
        PromoterAdapter(saveClickListener = PromoterAdapter.PromoterClickListener {
            viewModel.savePromoter(it)
        }, deleteClickListener = PromoterAdapter.PromoterClickListener {
            viewModel.deletePromoter(it.id)
        })


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        eventId = intent.getStringExtra(EVENT_ID).toString()
        _binding = ActivityHostDetailEventBinding.inflate(layoutInflater)
        setContentView(binding.root)
        title = "Event Detail"
        actionBar?.setDisplayHomeAsUpEnabled(true)
        initUi()
        initObservers()
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.host_detail_event_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.edit -> {
            setEditUi()
            true
        }
        R.id.my_account -> {
            onBackPressed()
            true
        }
        R.id.booking_list -> {
            navigateToBookingListActivity(viewModel.event.value?.id)
            true
        }
        R.id.check_in -> {
            navigateToCheckInActivity(viewModel.event.value?.id)
            true
        }

        else -> {
            super.onOptionsItemSelected(item)
        }
    }

    private fun initUi() {
        viewModel.initPopulateUI(eventId)
        setNonEditUi()

        // recycler views
        binding.rvTickets.layoutManager = LinearLayoutManager(this)
        binding.rvTickets.apply {
            adapter = ticketAdapter
        }

        binding.rvPromoters.layoutManager = LinearLayoutManager(this)
        binding.rvPromoters.apply {
            adapter = promoterAdapter
            setHasFixedSize(false)
            isNestedScrollingEnabled = false
        }

        // Listeners
        binding.ivPhotoIcon.setOnClickListener {
            launcher.launch("image/*")
        }

        binding.tvStartDate.setOnClickListener {
            showStartDatePicker(viewModel.startDate.value ?: Date())
        }

        binding.tvStartTime.setOnClickListener {
            showStartDatePicker(viewModel.startDate.value ?: Date())
        }

        binding.tvEndDate.setOnClickListener {
            showEndDatePicker(viewModel.endDate.value ?: Date())
        }

        binding.tvEndTime.setOnClickListener {
            showEndDatePicker(viewModel.endDate.value ?: Date())
        }

        binding.btnAddTicket.setOnClickListener {
            viewModel.addTicket(
                Ticket(
                    "tk-${viewModel.ticketsListSize() + 1}", null, null, null, null, true
                )
            )
        }
        binding.btnAddPromoter.setOnClickListener {
            viewModel.addPromoter(
                Promoter("prm-${viewModel.promotersListSize() + 1}", null, null, null, null, true)
            )
        }

        binding.btnSaveDraft.setOnClickListener { _ ->
            viewModel.event.value?.let { it ->
                val event = Event(
                    name = title.toString(),
                    description = binding.etDescription.text.toString(),
                    host = AppSarau.prefs.getUser(),
                    address = Address(
                        street = binding.etAddress.text.toString(),
                        postalCode = binding.etPostalCode.text.toString(),
                        city = binding.etCity.text.toString(),
                        country = binding.etCountry.text.toString()
                    ), startDate = viewModel.startDate.value ?: Date(0),
                    endDate = viewModel.endDate.value ?: Date(0),
                    tags = listOf(), published = false,
                    image = viewModel.imageTempUri.value.toString(),
                    paymentMethod = PaymentMethod(
                        name = binding.etPaymentMethodName.text.toString(),
                        description = binding.etPaymentMethodDescription.text.toString(),
                        account = binding.etPaymentMethodNum.text.toString()
                    ),
                    id = it.id
                )
                viewModel.validateDraft(event)
            }
        }

        binding.btnPublish.setOnClickListener { _ ->
            viewModel.event.value?.let { it ->
                val event = Event(
                    name = title.toString(),
                    description = binding.etDescription.text.toString(),
                    host = AppSarau.prefs.getUser(),
                    address = Address(
                        street = binding.etAddress.text.toString(),
                        postalCode = binding.etPostalCode.text.toString(),
                        city = binding.etCity.text.toString(),
                        country = binding.etCountry.text.toString()
                    ),
                    viewModel.startDate.value ?: Date(0),
                    viewModel.endDate.value ?: Date(0),
                    listOf(),
                    true,
                    viewModel.imageTempUri.value.toString(),
                    PaymentMethod(
                        name = binding.etPaymentMethodName.text.toString(),
                        description = binding.etPaymentMethodDescription.text.toString(),
                        account = binding.etPaymentMethodNum.text.toString()
                    ),
                    id = it.id
                )
                viewModel.validatePublish(event)
            }
        }
    }


    private fun initObservers() {
        viewModel.event.observe(this) { itEvent ->
            populateUi(itEvent)
            setIsPublished(itEvent)
            viewModel.setLocalUri(itEvent)
            viewModel.setInitialDates(itEvent)
            binding.scrollContainer.fullScroll(ScrollView.FOCUS_UP)
        }

        viewModel.initialTickets.observe(this) {
            viewModel.setTicketsList(it)
        }

        viewModel.initialPromoters.observe(this) { list ->
            viewModel.setPromotersList(list)
        }

        viewModel.tickets.observe(this) {
            ticketAdapter.submitList(it.filter { ticket -> ticket.active })
        }

        viewModel.promoters.observe(this) {
            promoterAdapter.submitList(it.filter { promoter -> promoter.active })
        }

        viewModel.imageTempUri.observe(this) {
            Glide.with(this).load(it).fitCenter().into(binding.ivPoster)
            convertToBitmap(it)
        }

        viewModel.startDate.observe(this) { date ->
            date?.let {
                val timeZone = TimeZone.getTimeZone("GMT")
                val dateFormat = SimpleDateFormat("dd/MM/yyy", Locale.getDefault())
                val timeFormat = SimpleDateFormat("HH:mm", Locale.getDefault())
                dateFormat.timeZone = timeZone
                timeFormat.timeZone = timeZone

                binding.tvStartDate.text = dateFormat.format(it)
                binding.tvStartTime.text = timeFormat.format(it)
            }
        }

        viewModel.endDate.observe(this) { date ->
            date?.let {
                val timeZone = TimeZone.getTimeZone("GMT")
                val dateFormat = SimpleDateFormat("dd/MM/yyy", Locale.getDefault())
                val timeFormat = SimpleDateFormat("HH:mm", Locale.getDefault())
                dateFormat.timeZone = timeZone
                timeFormat.timeZone = timeZone

                binding.tvEndDate.text = dateFormat.format(it)
                binding.tvEndTime.text = timeFormat.format(it)
            }
        }

        viewModel.errorMessage.observe(this) {
            if (it != null) {
                viewModel.resetErrorMessage()
                AlertDialog.Builder(this)
                    .setTitle(R.string.error)
                    .setMessage(it)
                    .setPositiveButton("Ok") { dialog, _ -> dialog.dismiss() }
                    .create()
                    .show()
            }
        }

        viewModel.validatedDraft.observe(this) {
            if (it != null) {
                viewModel.resetValidatedDraft()
                viewModel.updateEvent(it)
            }
        }

        viewModel.validatedPublish.observe(this) {
            if (it != null) {
                viewModel.resetValidatePublish()
                viewModel.updateEvent(it)
            }
        }

        viewModel.isEventUpdated.observe(this) {
            if (it) {
                Toast.makeText(this, getString(R.string.update_event_ok_message), Toast.LENGTH_SHORT)
                    .show()
                viewModel.refreshUI(eventId)
                setNonEditUi()
            }
        }
    }

    private fun setIsPublished(event: Event?) {
        event?.let {
            if (it.published) {
                binding.tvIsPublishedEvent.apply {
                    text = context.getString(R.string.event_published)
                    setTextColor(
                        ContextCompat.getColor(
                            applicationContext,
                            R.color.green
                        )
                    )
                }
            } else {
                binding.tvIsPublishedEvent.apply {
                    text = context.getString(R.string.event_not_published)
                    setTextColor(ContextCompat.getColor(applicationContext, R.color.red))
                }
            }
        }
    }

    private fun setNonEditUi() {
        binding.apply {
            etAddress.isEnabled = false
            etPostalCode.isEnabled = false
            etCity.isEnabled = false
            etCountry.isEnabled = false
            etDescription.isEnabled = false
            etPaymentMethodName.isEnabled = false
            etPaymentMethodDescription.isEnabled = false
            etPaymentMethodNum.isEnabled = false

            tvStartDate.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.text))
                isClickable = false
                isEnabled = false
            }
            tvStartTime.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.text))
                isClickable = false
                isEnabled = false
            }
            tvEndDate.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.text))
                isClickable = false
                isEnabled = false
            }
            tvEndTime.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.text))
                isClickable = false
                isEnabled = false
            }

            ivPhotoIcon.visibility = View.GONE
            btnAddPromoter.visibility = View.GONE
            btnAddTicket.visibility = View.GONE
            btnPublish.visibility = View.GONE
            btnSaveDraft.visibility = View.GONE
        }
    }

    private fun setEditUi() {
        binding.apply {
            etAddress.isEnabled = true
            etPostalCode.isEnabled = true
            etCity.isEnabled = true
            etCountry.isEnabled = true
            etDescription.isEnabled = true
            etPaymentMethodName.isEnabled = true
            etPaymentMethodDescription.isEnabled = true
            etPaymentMethodNum.isEnabled = true

            tvStartDate.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.orange))
                isClickable = true
                isEnabled = true
            }
            tvStartTime.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.orange))
                isClickable = true
                isEnabled = true
            }
            tvEndDate.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.orange))
                isClickable = true
                isEnabled = true
            }
            tvEndTime.apply {
                setTextColor(ContextCompat.getColor(applicationContext, R.color.orange))
                isClickable = true
                isEnabled = true
            }

            ivPhotoIcon.visibility = View.VISIBLE
            btnAddPromoter.visibility = View.VISIBLE
            btnAddTicket.visibility = View.VISIBLE
            btnPublish.visibility = View.VISIBLE
            btnSaveDraft.visibility = View.VISIBLE
        }
    }

    private fun populateUi(event: Event) {
        title = event.name
        binding.apply {
            etAddress.setText(event.address.street)
            etPostalCode.setText(event.address.postalCode)
            etCity.setText(event.address.city)
            etCountry.setText(event.address.country)
            etDescription.setText(event.description)
            etPaymentMethodName.setText(event.paymentMethod.name)
            etPaymentMethodDescription.setText(event.paymentMethod.description)
            etPaymentMethodNum.setText(event.paymentMethod.account)

            val startDate = Utils.setDateTime(event.startDate)
            tvStartDate.text = startDate.first
            tvStartTime.text = startDate.second

            val endDate = Utils.setDateTime(event.endDate)
            tvEndDate.text = endDate.first
            tvEndTime.text = endDate.second

            event.image?.let {
                Glide.with(this@HostDetailEventActivity).load(it).fitCenter().into(ivPoster)
            }
        }
    }

    private fun convertToBitmap(imageUri: Uri) {
        // Only create bitmap if is Uri of local content
        if (imageUri.toString().contains("content://")) {
            val bitmap = Utils.convertToBitmap(applicationContext, imageUri)
            viewModel.setPosterBitmap(bitmap)
        }
    }

    private fun showStartDatePicker(date: Date) {
        val datePicker = MaterialDatePicker.Builder.datePicker()
            .setSelection(date.time)
            .setTitleText(getString(R.string.select_start_date)).build()

        datePicker.show(supportFragmentManager, "MATERIAL_DATE_PICKER")

        datePicker.addOnPositiveButtonClickListener {
            viewModel.setStartDate(it)
            showStartTimePicker()
        }

    }

    private fun showStartTimePicker() {
        val timePicker =
            MaterialTimePicker.Builder().setTitleText(getString(R.string.select_start_time))
                .setTimeFormat(TimeFormat.CLOCK_24H).build()
        timePicker.show(supportFragmentManager, "CreateEventActivity")

        timePicker.addOnPositiveButtonClickListener {
            viewModel.setStartTime(timePicker.hour, timePicker.minute)
        }
    }

    private fun showEndDatePicker(date: Date) {
        val datePicker = MaterialDatePicker.Builder.datePicker()
            .setSelection(date.time)
            .setTitleText(getString(R.string.select_start_date)).build()

        datePicker.show(supportFragmentManager, "MATERIAL_DATE_PICKER")

        datePicker.addOnPositiveButtonClickListener {
            viewModel.setEndDate(it)
            showEndTimePicker()
        }

    }

    private fun showEndTimePicker() {
        val timePicker =
            MaterialTimePicker.Builder().setTitleText(getString(R.string.select_start_time))
                .setTimeFormat(TimeFormat.CLOCK_24H).build()
        timePicker.show(supportFragmentManager, "CreateEventActivity")

        timePicker.addOnPositiveButtonClickListener {
            viewModel.setEndTime(timePicker.hour, timePicker.minute)
        }
    }

    private fun navigateToCheckInActivity(eventId: String?) {
        val intent = Intent(baseContext, EventCheckInActivity::class.java)
        intent.putExtra(EventCheckInActivity.EVENT_ID, eventId)
        startActivity(intent)
    }

    private fun navigateToBookingListActivity(eventId: String?) {
        val intent = Intent(baseContext, BookingListActivity::class.java)
        intent.putExtra(BookingListActivity.EVENT_ID, eventId)
        startActivity(intent)
    }

}