package edu.uoc.bsolee.sarau.ui.host.main_host

import android.content.Intent
import android.graphics.ImageDecoder
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.provider.MediaStore
import android.view.Menu
import android.view.MenuItem
import android.view.View
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import com.bumptech.glide.Glide
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.LastEventHostAdapter
import edu.uoc.bsolee.sarau.adapters.NextEventHostAdapter
import edu.uoc.bsolee.sarau.databinding.ActivityMainHostBinding
import edu.uoc.bsolee.sarau.model.Address
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.model.Host
import edu.uoc.bsolee.sarau.ui.host.create_event.CreateEventActivity
import edu.uoc.bsolee.sarau.ui.host.detail_event.HostDetailEventActivity
import edu.uoc.bsolee.sarau.ui.login.LoginActivity


class MainHostActivity : AppCompatActivity() {


    // ViewModel
    private val viewModel: MainHostViewModel by viewModels { MainHostViewModelFactory() }

    // ViewBinding
    private var _binding: ActivityMainHostBinding? = null
    private val binding: ActivityMainHostBinding
        get() = _binding!!

    // Launchers
    private val launcher =
        registerForActivityResult(ActivityResultContracts.GetContent()) { result ->
            viewModel.selectImage(result)
        }

    // Adapters
    private val nextEventAdapter = NextEventHostAdapter(itemClickListener = NextEventHostAdapter.EventHostClickListener { event ->
        navigateToHostDetailActivity(event)
    })

    private val lastEventAdapter = LastEventHostAdapter(itemClickListener = LastEventHostAdapter.EventHostClickListener { event ->
        navigateToHostDetailActivity(event)
    })

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        _binding = ActivityMainHostBinding.inflate(layoutInflater)
        setContentView(binding.root)
        title = getString(R.string.my_account)
//        supportActionBar?.setDisplayHomeAsUpEnabled(true)
        initUi()
        initObservers()
        supportActionBar?.elevation = 0f
    }

    override fun onResume() {
        super.onResume()
        viewModel.getEvents()
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.main_host_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.edit -> {
            editProfile()
            true
        }

        R.id.logout -> {
            logout()
            true
        }

        R.id.create_event -> {
            navigateToCreateEventActivity()
            true
        }

        else -> {
            super.onOptionsItemSelected(item)
        }
    }

    private fun initUi() {
        // RecyclerViews
        binding.rvNextEvents.layoutManager = LinearLayoutManager(this,LinearLayoutManager.HORIZONTAL, false)
        binding.rvNextEvents.adapter = nextEventAdapter

        binding.rvLastEvents.layoutManager = LinearLayoutManager(this,LinearLayoutManager.HORIZONTAL, false)
        binding.rvLastEvents.adapter = lastEventAdapter

        // Listeners
        binding.btnCreateEvent.setOnClickListener {
            navigateToCreateEventActivity()
        }

        binding.btnSaveProfile.setOnClickListener {
            saveProfile()
        }

        binding.btnDiscard.setOnClickListener {
            discardChanges()
        }

        binding.ivPhotoIcon.setOnClickListener {
            launcher.launch("image/*")
        }
    }

    private fun initObservers() {
        viewModel.host.observe(this) {
            if (it != null) {
                populateProfile(it)
            }
        }

        viewModel.nextEvents.observe(this){
            nextEventAdapter.submitList(it)
        }

        viewModel.lastEvents.observe(this){
            lastEventAdapter.submitList(it)
        }

        viewModel.uriImage.observe(this) {
            setProfileImage(it)
        }

        viewModel.imageUrl.observe(this) {
            viewModel.updateImageOnDB(it)
        }

        viewModel.isLoggedOutDone.observe(this) {
            if (it) {
                viewModel.clearData()
                // Google Sign Out
                val gso = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                    .requestEmail()
                    .build()
                val googleSignInClient = GoogleSignIn.getClient(this, gso)
                googleSignInClient.signOut().addOnCompleteListener {
                    navigateToLoginActivity()
                }
            }
        }
    }

    private fun populateProfile(host: Host) {
        binding.apply {
            etContactName.setText(host.contactName)
            etBusinessName.setText(host.businessName)
            etEmail.setText(host.email)
            etTelephone.setText(host.telephone)
            host.address?.let {
                etStreet.setText(it.street)
                etPostalCode.setText(it.postalCode)
                etCity.setText(it.city)
                etCountry.setText(it.country)
            }
            etVat.setText(host.vatNumber)

            viewModel.selectImage(Uri.parse(host.image))
        }

    }

    private fun navigateToCreateEventActivity() {
        val intent = Intent(baseContext, CreateEventActivity::class.java)
        startActivity(intent)
    }

    private fun navigateToLoginActivity() {
        val intent = Intent(baseContext, LoginActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun logout() {
        viewModel.logout()
    }

    private fun editProfile() {
        binding.apply {
            etBusinessName.isEnabled = true
            etContactName.isEnabled = true
            etTelephone.isEnabled = true
            etStreet.isEnabled = true
            etPostalCode.isEnabled = true
            etCity.isEnabled = true
            etCountry.isEnabled = true
            etVat.isEnabled = true
            ivPhotoIcon.visibility = View.VISIBLE
            btnSaveProfile.visibility = View.VISIBLE
            btnDiscard.visibility = View.VISIBLE
            btnCreateEvent.visibility = View.GONE
            eventsContainer.visibility = View.GONE
        }
    }

    private fun saveProfile() {
        binding.apply {
            etBusinessName.isEnabled = false
            etContactName.isEnabled = false
            etTelephone.isEnabled = false
            etStreet.isEnabled = false
            etPostalCode.isEnabled = false
            etCity.isEnabled = false
            etCountry.isEnabled = false
            etVat.isEnabled = false
            ivPhotoIcon.visibility = View.GONE
            btnSaveProfile.visibility = View.GONE
            btnDiscard.visibility = View.GONE
            btnCreateEvent.visibility = View.VISIBLE
            eventsContainer.visibility = View.VISIBLE
        }

        val currentHostId = viewModel.host.value?.id
        val imageUrl = viewModel.host.value?.image

        viewModel.saveHostToDB(
            Host(
                currentHostId ?: "",
                binding.etBusinessName.text.toString(),
                binding.etContactName.text.toString(),
                binding.etEmail.text.toString(),
                binding.etTelephone.text.toString(),
                Address(
                    binding.etStreet.text.toString(),
                    binding.etPostalCode.text.toString(),
                    binding.etCity.text.toString(),
                    binding.etCountry.text.toString()
                ),
                binding.etVat.text.toString(),
                imageUrl
            )
        )
        // If image has been changed, save it to Firebase Storage and Firestore Database
        if (imageUrl != viewModel.uriImage.value.toString()) {
            viewModel.uriImage.value?.let { initSaveImageToDB(it) }
        }
    }

    private fun initSaveImageToDB(imageUri: Uri) {
        val bitmap = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
            ImageDecoder.decodeBitmap(
                ImageDecoder.createSource(
                    applicationContext.contentResolver,
                    imageUri
                )
            )
        } else {
            MediaStore.Images.Media.getBitmap(applicationContext.contentResolver, imageUri)
        }
        viewModel.saveImageToDB(bitmap)
    }

    private fun discardChanges() {
        binding.apply {
            etBusinessName.isEnabled = false
            etContactName.isEnabled = false
            etTelephone.isEnabled = false
            etStreet.isEnabled = false
            etPostalCode.isEnabled = false
            etCity.isEnabled = false
            etCountry.isEnabled = false
            etVat.isEnabled = false
            ivPhotoIcon.visibility = View.GONE
            btnSaveProfile.visibility = View.GONE
            btnDiscard.visibility = View.GONE
            btnCreateEvent.visibility = View.VISIBLE
            eventsContainer.visibility = View.VISIBLE
        }
        viewModel.getCurrentHost()
    }

    private fun setProfileImage(uri: Uri) {
        Glide.with(applicationContext).load(uri).centerCrop().into(binding.ivImage)
    }

    private fun navigateToHostDetailActivity(event: Event) {
        val intent = Intent(baseContext, HostDetailEventActivity::class.java)
        intent.putExtra(HostDetailEventActivity.EVENT_ID, event.id)
        startActivity(intent)
    }


}