package edu.uoc.bsolee.sarau.ui.user.booking_detail

import android.annotation.SuppressLint
import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.zxing.BarcodeFormat
import com.journeyapps.barcodescanner.BarcodeEncoder
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.BookDetailTicketAdapter
import edu.uoc.bsolee.sarau.databinding.ActivityBookingDetailBinding
import edu.uoc.bsolee.sarau.ui.user.main_user.MainUserActivity
import edu.uoc.bsolee.sarau.ui.user.my_account.UserAccountActivity
import edu.uoc.bsolee.sarau.utils.Utils
import java.text.NumberFormat
import java.util.*


class BookingDetailActivity : AppCompatActivity() {

    companion object {
        const val BOOKING_QR = "bookingQR"
    }

    // Argument
    private lateinit var bookingQR: String

    // View Binding
    private var _binding: ActivityBookingDetailBinding? = null
    val binding: ActivityBookingDetailBinding get() = _binding!!

    // ViewModel
    private val viewModel: BookingDetailViewModel by viewModels { BookingDetailViewModelFactory() }

    // Adapter
    private lateinit var ticketAdapter: BookDetailTicketAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        bookingQR = intent.getStringExtra(BOOKING_QR).toString()
        actionBar?.setDisplayHomeAsUpEnabled(true)
        _binding = ActivityBookingDetailBinding.inflate(layoutInflater)
        initUI()
        initObservers()
        setContentView(binding.root)
    }

    override fun onResume() {
        super.onResume()
        viewModel.setBookingQR(bookingQR)
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.user_event_detail, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.events -> {
            navigateToEventsList()
            true
        }
        R.id.my_account -> {
            navigateToMyAccount()
            true
        }
        else -> {
            super.onOptionsItemSelected(item)
        }
    }

    private fun initUI() {
        // recycler View
        ticketAdapter = BookDetailTicketAdapter()
        binding.rvTickets.layoutManager = LinearLayoutManager(this)
        binding.rvTickets.adapter = ticketAdapter

        // Listeners
        binding.tvPaymentAccount.setOnClickListener {
            val textView = it as TextView
            copyToClipBoard(textView.text.toString().substring(5))
        }
    }

    private fun initObservers() {
        viewModel.booking.observe(this) {
            viewModel.getEventInfo(it.eventId)
            viewModel.getBookedTickets(it)
        }

        viewModel.event.observe(this) {
            populateUI()
        }

        viewModel.bookedTickets.observe(this) {
            if (it.isNotEmpty()) {
                ticketAdapter.submitList(it.filter { ticket -> ticket.num > 0 })
            }
        }

        viewModel.accountCopied.observe(this) {
            if (it) {
                viewModel.onAccountCopiedDone()
                Toast.makeText(this, getString(R.string.account_copied), Toast.LENGTH_SHORT).show()
            }
        }
    }

    @SuppressLint("SetTextI18n")
    private fun populateUI() {
        val booking =
            viewModel.booking.value!! //populate UI after get booking and event, so not null
        val event = viewModel.event.value!!
        title = event.name
        // QR CODE
        if (booking.paid) {
            binding.apply {
                ivQr.visibility = View.VISIBLE
                tvPendingPaymentQr.visibility = View.GONE
                tvPaymentStatus.text = getString(R.string.payment_confirmed)
                tvPaymentStatus.setTextColor(ContextCompat.getColor(baseContext, R.color.green))
                generateQR(booking.qr)
            }
        } else {
            binding.apply {
                ivQr.visibility = View.GONE
                tvPendingPaymentQr.visibility = View.VISIBLE
                tvPaymentStatus.text = getString(R.string.pending_payment_confirmation)
                tvPaymentStatus.setTextColor(ContextCompat.getColor(baseContext, R.color.red))
            }
        }
        // Payment info
        val numFormat = NumberFormat.getCurrencyInstance(Locale.FRANCE)
        binding.apply {
            tvPromoterCode.text =
                booking.let { getString(R.string.promoter_code) + ": ${it.promoterCode}" }
            tvPaymentName.text = event.paymentMethod.name
            tvPaymentDescription.text = event.paymentMethod.description
            tvPaymentAmount.text =
                getString(R.string.amount) + " " + numFormat.format(booking.amount)
            tvPaymentAccount.text = "NUM: " + event.paymentMethod.account
            tvPaymentRef.text = "REF: ${booking.id} - ${event.name}"
        }

        // Event info
        binding.apply {
            tvCity.text = event.address.city
            tvAddress.text = event.address.street
            tvAddressOther.text =
                event.address.let { "${it.postalCode}, ${it.city}, ${it.country}" }
            tvStartDate.text = Utils.setDateTime(event.startDate).first
            tvStartTime.text = Utils.setDateTime(event.startDate).second
            tvEndDate.text = Utils.setDateTime(event.endDate).first
            tvEndTime.text = Utils.setDateTime(event.endDate).second
        }
    }


    private fun navigateToMyAccount() {
        val intent = Intent(baseContext, UserAccountActivity::class.java)
        startActivity(intent)
    }

    private fun navigateToEventsList() {
        val intent = Intent(baseContext, MainUserActivity::class.java)
        startActivity(intent)
    }

    private fun copyToClipBoard(account: String) {
        val clipboard = getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
        val clip: ClipData = ClipData.newPlainText("account", account)
        clipboard.setPrimaryClip(clip)
        viewModel.onAccountCopied()
    }

    private fun generateQR(qrString: String) {
        try {
            val barcodeEncoder = BarcodeEncoder()
            val bitmap = barcodeEncoder.encodeBitmap(qrString, BarcodeFormat.QR_CODE, 400, 400)
            binding.ivQr.setImageBitmap(bitmap)
        } catch (e: Exception) {
        }
    }
}