package edu.uoc.bsolee.sarau.ui.user.main_user

import android.annotation.SuppressLint
import android.app.AlertDialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.transition.TransitionInflater
import android.view.*
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.activity.OnBackPressedCallback
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.recyclerview.widget.LinearLayoutManager
import com.bumptech.glide.Glide
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.BookTicketAdapter
import edu.uoc.bsolee.sarau.databinding.FragmentEventDetaillUserBinding
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.Event
import edu.uoc.bsolee.sarau.ui.user.booking_detail.BookingDetailActivity
import edu.uoc.bsolee.sarau.ui.user.my_account.UserAccountActivity
import edu.uoc.bsolee.sarau.utils.Utils


class EventDetailUserFragment : Fragment() {
    companion object {
        const val EVENT_ID_PARAM = "eventId"

        @JvmStatic
        fun newInstance(eventId: String) = EventDetailUserFragment().apply {
            arguments = Bundle().apply { putString(EVENT_ID_PARAM, eventId) }
        }
    }

    private var eventId: String? = null

    // ViewModel
    private val viewModel: MainUserViewModel by activityViewModels { MainUserViewModelFactory() }

    // View Binding
    private var _binding: FragmentEventDetaillUserBinding? = null
    private val binding: FragmentEventDetaillUserBinding
        get() = _binding!!

    // adapter
    private lateinit var ticketAdapter: BookTicketAdapter


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            eventId = it.getString(EVENT_ID_PARAM)
        }
        setHasOptionsMenu(true)
        val inflater = TransitionInflater.from(requireContext())
        enterTransition = inflater.inflateTransition(R.transition.slide_right)
        exitTransition = inflater.inflateTransition(R.transition.fade)
    }

    override fun onResume() {
        super.onResume()
        //  reset UI
        resetUI()
        viewModel.getEvent(eventId)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentEventDetaillUserBinding.inflate(inflater, container, false)
        val view = binding.root
        initUi()
        initObservers()
        return view
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        activity?.onBackPressedDispatcher?.addCallback(viewLifecycleOwner,
            object : OnBackPressedCallback(true) {
                override fun handleOnBackPressed() {
                    viewModel.reset()
                    viewModel.setFragment(MainUserActivity.EVENTS_FRAGMENT)
                }
            })
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onCreateOptionsMenu(menu: Menu, inflater: MenuInflater) {
        inflater.inflate(R.menu.user_event_detail, menu)
        super.onCreateOptionsMenu(menu, inflater)
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.events -> {
            navigateToEventsList()
            true
        }
        R.id.my_account -> {
            navigateToMyAccount()
            true
        }
        else -> {
            super.onOptionsItemSelected(item)
        }
    }

    private fun initUi() {
        ticketAdapter =
            BookTicketAdapter(
                requireContext(),
                BookTicketAdapter.TicketSelectorListener { numTickets, ticket ->
                    viewModel.chooseTickets(Pair(numTickets, ticket))
                })
        // recyclerview
        binding.rvTickets.layoutManager = LinearLayoutManager(context)
        binding.rvTickets.adapter = ticketAdapter

        // Listeners
        binding.btnBookTicket.setOnClickListener {
            viewModel.initBooking()
        }

        // setup return key action
        binding.etPromoterCode.setOnEditorActionListener { textView, actionId, _ ->
            var action = false
            if (actionId == EditorInfo.IME_ACTION_DONE) {
                textView.clearFocus()
                // hide keyboard
                val inputMethodManager: InputMethodManager =
                    textView.context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                inputMethodManager.hideSoftInputFromWindow(textView.windowToken, 0)
                viewModel.checkPromoterCode(textView.text.toString())
                action = true
            }
            return@setOnEditorActionListener action
        }
    }

    private fun initObservers() {
        viewModel.eventDetail.observe(viewLifecycleOwner) {
            //Populate UI with event info
            populateUI(it)
        }

        viewModel.eventTickets.observe(viewLifecycleOwner) {
            ticketAdapter.submitList(it)
        }

        viewModel.initBooking.observe(viewLifecycleOwner) {
            if (it) {
                viewModel.checkSelectedTickets()
            }
        }

        viewModel.totalTickets.observe(viewLifecycleOwner) {
            if (it > 0) {
                viewModel.checkPromoterCode(binding.etPromoterCode.text.toString())
            } else {
                viewModel.stopBooking()
                showNoTicketsDialog()
            }
        }

        viewModel.promoter.observe(viewLifecycleOwner) { promoter ->
            promoter?.let {
                if (it.isNotEmpty() && it != viewModel.defaultPromoterCode) {
                    binding.tvPromoterMessage.apply {
                        visibility = View.VISIBLE
                        setText(R.string.promoter_code_ok_message)
                        setTextColor(ContextCompat.getColor(requireContext(), R.color.green))
                        viewModel.initCheckAvailableStock()
                    }
                } else if (it == viewModel.defaultPromoterCode) {
                    viewModel.initCheckAvailableStock().also {
                        binding.tvPromoterMessage.visibility = View.GONE
                    } // for empty code
                } else {
                    viewModel.stopBooking()
                    binding.tvPromoterMessage.apply {
                        visibility = View.VISIBLE
                        setText(R.string.promoter_code_ko_message)
                        setTextColor(ContextCompat.getColor(requireContext(), R.color.red))
                        Toast.makeText(context, R.string.check_promoter_code, Toast.LENGTH_SHORT)
                            .show()
                    }
                }
            }
        }

        viewModel.stockProblems.observe(viewLifecycleOwner) {
            if (it != null) {
                viewModel.resetStockProblems()
                if (it.isNotEmpty()) {
                    showStockProblemsDialog(it)
                } else {
                    viewModel.checkPromoterBookings(
                        binding.etPromoterCode.text.toString().uppercase()
                    )
                }
            }
        }

        viewModel.promoterBookings.observe(viewLifecycleOwner) { list ->
            viewModel.initSaveBooking(list.size + 1)
        }

        viewModel.bookingSaved.observe(viewLifecycleOwner) { booking ->
            booking?.let { navigateToBookingDetail(it)}
        }

    }


    private fun resetUI() {
        activity?.title = ""
        binding.apply {
            tvCity.text = ""
            tvStreet.text = ""
            tvAddressOther.text = ""
            tvStartDate.text = ""
            tvStartTime.text = ""
            tvEndDate.text = ""
            tvEndTime.text = ""
            tvDescription.text = ""

            Glide.with(this@EventDetailUserFragment)
                .load(ContextCompat.getDrawable(requireContext(), R.drawable.sarau)).into(ivPoster)
        }
    }

    @SuppressLint("SetTextI18n")
    private fun populateUI(event: Event) {
        activity?.title = event.name
        binding.apply {
            tvCity.text = event.address.city
            tvStreet.text = event.address.street
            tvAddressOther.text =
                "${event.address.postalCode} ${event.address.city}, ${event.address.country}"
            tvStartDate.text = Utils.setDateTime(event.startDate).first
            tvStartTime.text = Utils.setDateTime(event.startDate).second
            tvEndDate.text = Utils.setDateTime(event.endDate).first
            tvEndTime.text = Utils.setDateTime(event.endDate).second
            tvDescription.text = event.description

            event.image?.let {
                Glide.with(this@EventDetailUserFragment).load(it).into(ivPoster)
            }
        }
    }

    private fun showStockProblemsDialog(problems: List<Pair<Int, String>>) {
        val message = StringBuilder()
        for (problem in problems) {
            message.append(problem.second)
            message.append(" only has ")
            message.append(problem.first)
            message.append(" tickets available \n")
        }
        val builder = AlertDialog.Builder(context).setTitle("Sorry").setMessage(message)
            .setPositiveButton(R.string.ok) { dialog, _ ->
                dialog.dismiss()
            }
        builder.create().show()

    }

    private fun showNoTicketsDialog() {
        val message = getString(R.string.select_at_least_one_ticket)
        val builder = AlertDialog.Builder(context).setTitle("Error").setMessage(message)
            .setPositiveButton(R.string.ok) { dialog, _ ->
                dialog.dismiss()
            }
        builder.create().show()

    }

    private fun navigateToMyAccount() {
        viewModel.reset()
        val intent = Intent(requireContext(), UserAccountActivity::class.java)
        startActivity(intent)
    }

    private fun navigateToEventsList() {
        viewModel.reset()
        viewModel.setFragment(MainUserActivity.EVENTS_FRAGMENT)
    }

    private fun navigateToBookingDetail(booking: Booking) {
        viewModel.reset()
        val intent = Intent(requireContext(),BookingDetailActivity::class.java)
        intent.putExtra(BookingDetailActivity.BOOKING_QR, booking.qr)
        startActivity(intent)
    }


}