package edu.uoc.bsolee.sarau.ui.user.main_user

import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import edu.uoc.bsolee.sarau.AppSarau
import edu.uoc.bsolee.sarau.model.Booking
import edu.uoc.bsolee.sarau.model.Promoter
import edu.uoc.bsolee.sarau.model.SelectedTicket
import edu.uoc.bsolee.sarau.model.Ticket
import edu.uoc.bsolee.sarau.repository.BookingRepository
import edu.uoc.bsolee.sarau.repository.EventRepository
import edu.uoc.bsolee.sarau.repository.UserRepository
import java.util.*

class MainUserViewModel(
    private val eventRepository: EventRepository,
    private val userRepository: UserRepository,
    private val bookingRepository: BookingRepository
) : ViewModel() {

    val defaultPromoterCode = "NOPROMOTER"

    private val defaultPromoter = Promoter(
        "prm-1", "Default_Promoter", null, null, defaultPromoterCode, false
    )

    val nextEvents = eventRepository.getNextUserEventsLiveData()

    private val _fragment = MutableLiveData<Int>()
    val fragment: LiveData<Int>
        get() = _fragment

    private var _eventId = ""
    val eventId: String
        get() = _eventId

    val eventDetail = eventRepository.getEventLiveData()
    val currentUser = userRepository.getUserLiveData()
    val eventTickets = eventRepository.getTicketsLiveData()
    val eventPromoters = eventRepository.getPromotersLiveData()
    val promoterBookings = bookingRepository.getBookingsFromPromoterLiveData()
    val bookingSaved = userRepository.getBookingSaved()

    private val _promoter = MutableLiveData<String?>(null)
    val promoter: LiveData<String?> get() = _promoter
    var promoterId = ""

    private var _selectedTickets: MutableList<SelectedTicket> = mutableListOf()
    val selectedTickets: List<SelectedTicket> get() = _selectedTickets

    private var _totalTickets = MutableLiveData<Int>()
    val totalTickets : LiveData<Int> get() = _totalTickets

    private var _stockProblems = MutableLiveData<List<Pair<Int, String>>?>(null)
    val stockProblems: LiveData<List<Pair<Int, String>>?> get() = _stockProblems

    private val _initBooking = MutableLiveData(false)
    val initBooking: LiveData<Boolean> get() = _initBooking

    fun initBooking() {
        _initBooking.value = true
    }

    fun stopBooking() {
        _initBooking.value = false
    }

    fun setFragment(fragment: Int) {
        _fragment.value = fragment
    }

    fun setEventId(id: String) {
        _eventId = id
    }

    fun getEvent(eventId: String?) {
        eventId?.let { eventRepository.getEventFromId(it) }
    }

    fun chooseTickets(numTicket: Pair<Int, Ticket>) {
        val ticket = SelectedTicket(numTicket.first, numTicket.second)
        val newList =
            selectedTickets.filter { item -> numTicket.second.id != item.ticket.id } as MutableList<SelectedTicket>
        newList.add(ticket)
        _selectedTickets = newList
        for (pair in selectedTickets) {
            Log.d("MainUserViewModel", "num: ${pair.num} - ticket: ${pair.ticket.price}")
        }
    }


    fun checkSelectedTickets(){
        if(initBooking.value == true) {
            var numTickets = 0
            selectedTickets.map { ticket ->
                numTickets += ticket.num
            }
            _totalTickets.value = numTickets
        }

    }

    fun checkPromoterCode(code: String) {
        if (code.isNotEmpty()) {
            val promoters = eventPromoters.value
            promoters?.forEach { promoter ->
                if (promoter.code?.uppercase() == code.uppercase()) { // promoter validated
                    _promoter.value = code.uppercase()
                    promoterId = promoter.id
                    return
                }
            }
            _promoter.value = "" // promoter no validated
        } else {
            // empty promoter -> default promoter code
            _promoter.value = defaultPromoterCode
            promoterId = defaultPromoter.id
        }
    }

    fun initCheckAvailableStock() {
        if (initBooking.value == true) {
            checkAvailableStock()
        }
    }

    private fun checkAvailableStock() {
        val tickets = eventTickets.value
        val problems: MutableList<Pair<Int, String>> = mutableListOf()
        if (tickets != null) {
            for (ticket in tickets) {
                for (selectedTicked in selectedTickets) {
                    if (selectedTicked.ticket.id == ticket.id && selectedTicked.num > ticket.capacity!!) {
                        problems.add(Pair(ticket.capacity, ticket.name!!))
                    }
                }
            }
        }
        _stockProblems.value = problems
    }

    fun resetStockProblems() {
        _stockProblems.value = null
    }

    fun reset() {
        resetStockProblems()
        stopBooking()
        userRepository.resetBookingSaved()
        _promoter.value = null
        _selectedTickets = arrayListOf()
    }

    fun checkPromoterBookings(code: String?) {
        eventDetail.value?.let { event ->
            if (!code.isNullOrEmpty()) {
                //code has been validated previously
                val promoter = eventPromoters.value?.filter { promoter -> promoter.code?.uppercase() == code.uppercase() }
                if (promoter != null) {
                    bookingRepository.getBookingsFromPromoter(event, promoter[0])
                }
            } else {
                //No code promoter, promoter = default Promoter
                bookingRepository.getBookingsFromPromoter(event, defaultPromoter)
            }
        }
    }

    fun initSaveBooking(refNum: Int) {
        if(initBooking.value == true) {
            var amount = 0.0
            val description = StringBuilder()
            for (ticket in selectedTickets) {
                ticket.ticket.price?.let {
                    amount += ticket.num * it
                }
                ticket.ticket.name?.let {
                    description.append(ticket.num)
                    description.append("x")
                    description.append(it)
                    description.append("\n")
                }
            }

            val booking = Booking(
                id = "${promoter.value}-$refNum",
                description = description.toString(),
                name = currentUser.value?.name ?: "",
                email = currentUser.value?.email!!,
                phone = currentUser.value?.telephone ?: "",
                eventId = eventDetail.value?.id!!,
                event = eventDetail.value,
                promoterId = promoterId,
                promoterCode = promoter.value,
                selectedTickets = selectedTickets,
                qr = "${eventDetail.value?.id!!}${promoter.value}$refNum",
                amount = amount,
                paid = false,
                checkedIn = false,
                dateCreated = Date(),
                datePayment = null,
                dateCheckIn = null
            )
            bookingRepository.saveBookingToDB(booking)
            userRepository.saveBookingToUser(booking)
        }
    }

    init {
        eventRepository.getUsersNextEvents()
        userRepository.getUser(AppSarau.prefs.getUser())
    }

}