package edu.uoc.bsolee.sarau.ui.user.my_account

import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import com.bumptech.glide.Glide
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import edu.uoc.bsolee.sarau.R
import edu.uoc.bsolee.sarau.adapters.LastBookingsUserAdapter
import edu.uoc.bsolee.sarau.adapters.NextBookingsUserAdapter
import edu.uoc.bsolee.sarau.databinding.ActivityAccountUserBinding
import edu.uoc.bsolee.sarau.model.User
import edu.uoc.bsolee.sarau.ui.login.LoginActivity
import edu.uoc.bsolee.sarau.ui.user.booking_detail.BookingDetailActivity
import edu.uoc.bsolee.sarau.ui.user.main_user.MainUserActivity
import edu.uoc.bsolee.sarau.utils.Utils
import java.util.*

class UserAccountActivity : AppCompatActivity() {

    // View Binding
    private var _binding: ActivityAccountUserBinding? = null
    val binding: ActivityAccountUserBinding get() = _binding!!

    // ViewModel
    private val viewModel: UserAccountViewModel by viewModels { UserAccountViewModelFactory() }

    // Launchers
    private val launcher =
        registerForActivityResult(ActivityResultContracts.GetContent()) { result ->
            viewModel.selectImage(result)
        }

    // Adapters
    private val nextBookingsAdapter =
        NextBookingsUserAdapter(itemClickListener = NextBookingsUserAdapter.BookingUserClickListener { qr ->
            navigateToBookingDetailActivity(qr)
        })

    private val lastBookingsAdapter =
        LastBookingsUserAdapter(itemClickListener = LastBookingsUserAdapter.BookingUserClickListener { qr ->
            navigateToBookingDetailActivity(qr)
        })


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        _binding = ActivityAccountUserBinding.inflate(layoutInflater)
        title = getString(R.string.my_account)
        initUI()
        initObservers()
        setContentView(binding.root)
        supportActionBar?.elevation = 0f
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.user_account_menu, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem) = when (item.itemId) {
        R.id.edit -> {
            editProfile()
            true
        }

        R.id.logout -> {
            logout()
            true
        }

        R.id.events -> {
            navigateToEventsList()
            true
        }

        else -> {
            super.onOptionsItemSelected(item)
        }
    }

    private fun initUI() {
        // recycler view
        binding.rvNextEvents.layoutManager =
            LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false)
        binding.rvNextEvents.adapter = nextBookingsAdapter

        binding.rvLastEvents.layoutManager =
            LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false)
        binding.rvLastEvents.adapter = lastBookingsAdapter

        // Listeners
        binding.ivPhotoIcon.setOnClickListener {
            launcher.launch("image/*")
        }

        binding.btnSaveProfile.setOnClickListener {
            saveProfile()
        }

        binding.btnDiscard.setOnClickListener {
            discardChanges()
        }
    }

    private fun initObservers() {
        viewModel.user.observe(this) {
            populateUI(it)
            viewModel.selectImage(Uri.parse(it.image))
        }

        viewModel.bookings.observe(this) { list ->
            val nextList = list.filter { booking ->
                booking.event!!.startDate >= Date()
            }.sortedBy { booking -> booking.event!!.startDate }
            val lastList = list.filter { booking ->
                booking.event!!.startDate < Date()
            }.sortedBy { booking -> booking.event!!.startDate }

            nextBookingsAdapter.submitList(nextList)
            lastBookingsAdapter.submitList(lastList)
        }

        viewModel.uriImage.observe(this) {
            setProfileImage(it)
        }

        viewModel.userSavedSuccessful.observe(this) {
            if (it) {
                Toast.makeText(this, getString(R.string.user_saved), Toast.LENGTH_SHORT).show()
            }
        }

        viewModel.isLoggedOutDone.observe(this) {
            if (it) {
                // Clear SharedPreferences
                viewModel.clearData()
                // Google Sign Out
                val gso = GoogleSignInOptions.Builder(GoogleSignInOptions.DEFAULT_SIGN_IN)
                    .requestEmail()
                    .build()
                val googleSignInClient = GoogleSignIn.getClient(this, gso)
                googleSignInClient.signOut().addOnCompleteListener {
                    navigateToLoginActivity()
                }
            }
        }
    }

    private fun editProfile() {
        binding.apply {
            etUserName.isEnabled = true
            etUserPhone.isEnabled = true
            ivPhotoIcon.visibility = View.VISIBLE
            btnSaveProfile.visibility = View.VISIBLE
            btnDiscard.visibility = View.VISIBLE
        }
    }

    private fun populateUI(user: User) {
        binding.apply {
            etUserName.setText(user.name)
            etUserEmail.setText(user.email)
            etUserPhone.setText(user.telephone)
        }

        setProfileImage(Uri.parse(user.image))
    }

    private fun setProfileImage(uri: Uri) {
        Glide.with(this).load(uri).centerCrop().into(binding.ivImage)
    }

    private fun saveProfile() {
        binding.apply {
            etUserName.isEnabled = false
            etUserPhone.isEnabled = false
            ivPhotoIcon.visibility = View.GONE
            btnSaveProfile.visibility = View.INVISIBLE
            btnDiscard.visibility = View.INVISIBLE
        }

        val currentUserId = viewModel.user.value?.id
        val userEdited = User(
            currentUserId ?: "",
            binding.etUserName.text.toString(),
            binding.etUserEmail.text.toString(),
            binding.etUserPhone.text.toString(),
            viewModel.uriImage.value.toString()
        )

        // Check if image has benn updated
        if (viewModel.uriImage.value.toString() == viewModel.user.value?.image) {
            viewModel.updateUserToDB(userEdited)
        } else {
            val bitmap =
                viewModel.uriImage.value?.let { Utils.convertToBitmap(applicationContext, it) }
            bitmap?.let { viewModel.updateImageAndSaveUser(userEdited,it) }
        }
    }

    private fun discardChanges() {
        binding.apply {
            etUserName.isEnabled = false
            etUserPhone.isEnabled = false
            ivPhotoIcon.visibility = View.GONE
            btnSaveProfile.visibility = View.INVISIBLE
            btnDiscard.visibility = View.INVISIBLE
        }
        viewModel.getCurrentUser()
    }

    private fun logout() {
        viewModel.logout()
    }

    private fun navigateToBookingDetailActivity(qr: String) {
        val intent = Intent(baseContext, BookingDetailActivity::class.java)
        intent.putExtra(BookingDetailActivity.BOOKING_QR, qr)
        startActivity(intent)
    }


    private fun navigateToEventsList() {
        val intent = Intent(baseContext, MainUserActivity::class.java)
        startActivity(intent)
    }

    private fun navigateToLoginActivity() {
        val intent = Intent(baseContext, LoginActivity::class.java)
        startActivity(intent)
        finish()
    }

}